var updateRequest;

var KeyUpDir = "Images/keys/up/";
var KeyDownDir = "Images/keys/down/";

var lowestKeyNumber;
var highestKeyNumber;

var whites = new Array(0, 2, 4, 5, 7, 9, 11);  
var blacks = new Array(1, 3, 6, 8, 10);

var ascii = new Array(65, 87, 83, 69, 68, 70, 84, 71, 89, 72, 85, 74, 75, 79, 76, 80, 186, 222, 221, 13);

var MouseDown = false;

if (window.widget) {
    widget.onhide = onHide;
    widget.onshow = onShow;
}

document.addEventListener("keydown", keyDown, true);
document.addEventListener("keyup", keyUp, true);
document.addEventListener("keypress", keyKill, true);

// This object represents the state of the entire keyboard. We store whether a particular key
// is up or down. This will allow us to prevent key repeats from retriggering notes.
// (Unfortunately, WebKit treats a key repeat as a whole new keydown/keypress pair.)
var keyboard = new Object();

var keyShift = 0;         // shifts the key input. mouseclicks remain a=440
var transpose = 0;        // shifts the pitch of the entire UI
var keyShiftRaise = 12    // sets keyboard input for keyShift = 0 to the second octave of the piano
var pitchRaise = 4 * 12;  // brings piano into reasonable range for transpose = 0

function initialize() {
    MidiPlugin.acknowledge("MidiPlugin by Jared R. White (b. 8/11/1980). All rights reserved.");
    MidiPlugin.startAudio();
    checkForUpdate();
}

function loadSettings() {
    populateSynths();
    populateVoices();
}

function getSynthName(path) {
    var lastSlash = path.lastIndexOf("/");
    if (lastSlash == -1) return getTrimmedSynthName(path);
    else return getTrimmedSynthName(path.substring(lastSlash + 1));
}

function getTrimmedSynthName(name) {
    if (name.length < 32) return name;
    // assume we have a filetype suffix
    var lastPeriod = name.lastIndexOf(".");
    var namePart = name.substring(0, lastPeriod);
    var trimmedNamePart = namePart.substring(0, 29);
    if (trimmedNamePart.length < namePart.length) trimmedNamePart += "(...)";
    var suffix = name.substr(lastPeriod);
    return trimmedNamePart + suffix;
}

function populateSynths() {
    var synths = MidiPlugin.getSoundBanks();
    document.configuration.synth.options.length = 0;
    for (var i = 0; i < synths.length; i++) {
       document.configuration.synth.options[i] = new Option(getSynthName(synths[i]), synths[i]);
    }
}

function populateVoices() {
    var voices = MidiPlugin.getVoices();
    document.configuration.voice.options.length = 0;
    for (var i = 0; i < voices.length; i++) {
        document.configuration.voice.options[i] = new Option(voices[i], i);
    }
}

function keyToNote(asciiKey) {
    for (var i = 0; i < ascii.length; i++) if (asciiKey == ascii[i]) return i;
    return null;
}

function getNote(asciiKey) {
    var note = keyToNote(asciiKey);
    if (note == null) return null;
    return note + keyShift + keyShiftRaise;
}

function keyDown() {
    var note = getNote(event.keyCode);
    if (note != null) depress(note);
    keyKill();
}

function keyUp() {
    var note = getNote(event.keyCode);
    if (note != null) release(note);
    keyKill();
}

function keyKill() {
    event.stopPropagation();
    event.preventDefault();
}

function getColor(note) {
    // reduce note until it's in the first octave
    while (12 <= note) note -= 12;
    // look for that note in the black key array
    for (var i = 0; i < blacks.length; i++) if (note == blacks[i]) return 'black';
    return 'white';
}

function keyNumberToMidiNumber(keyNumber) {
    return keyNumber + transpose + pitchRaise;
}

function depress(keyNumber) {
    // if the note is already down, don't depress
    if (keyboard[keyNumber] == true) return;
    // otherwise, mark the key as down
    keyboard[keyNumber] = true;
    
    if (lowestKeyNumber <= keyNumber && keyNumber <= highestKeyNumber) document[keyNumber].src = KeyDownDir + getColor(keyNumber) + ".png";
    MidiPlugin.noteOn(keyNumberToMidiNumber(keyNumber));
}

function release(keyNumber) {
    // always mark the key as up -- we don't care if we get multiples
    keyboard[keyNumber] = false;    
    if (lowestKeyNumber <= keyNumber && keyNumber <= highestKeyNumber) document[keyNumber].src = KeyUpDir + getColor(keyNumber) + ".png";
    MidiPlugin.noteOff(keyNumberToMidiNumber(keyNumber));
}

function onShow() {
    MidiPlugin.startAudio();
}

function onHide() {
    MidiPlugin.stopAudio();
}

function mouseOver(note) {
	if (MouseDown) depress(note);
}

function mouseOut(note) {
	if (MouseDown) release(note);
}

function renderKey(note, color, x, y) {
	document.write("<a onMouseDown=\"depress(" + note + "); MouseDown = true\" onMouseUp=\"release(" + note + "); MouseDown = false\" onMouseOver=\"mouseOver(" + note + ")\" onMouseOut=\"mouseOut(" + note + ")\">");
	document.write("<img name='" + note + "' src='" + KeyUpDir + color + ".png' class='undraggable' style='position:absolute; LEFT: " + x + "px; TOP: " + y + "px;' />");
	document.write("</a>");
}

function renderWhiteKeys(x, y) {

    lowestKeyNumber = 0;
    highestKeyNumber = 12*3;
   
    for (var i = 0; i < 3; i++) for (var j = 0; j < whites.length; j++) {
        renderKey(whites[j] + 12*i, 'white', x, y);
        x += 24;
    }
    renderKey(12*3, 'white', x, y);    
}

function renderBlackKeys(x, y) {
	x += 16;
    for (var i = 0; i < 3; i++) for (var j = 0; j < blacks.length; j++) {
        renderKey(blacks[j] + 12*i, 'black', x, y);
        if (j == 1 || j == 4) x += 45;
        else x += 26;
    }
}

function handleShiftEvent(shiftAmount) {

    var pieces = event.srcElement.src.split("/");
    // the src is absolute
    var stepperDir = pieces[pieces.length - 2];

    var image;
    if (event.srcElement.src.indexOf("/up") != -1) image = "up"; else image = "down";

    if (event.type == "mouseup" || event.type == "mouseout") {
        event.srcElement.src = "Images/stepper/" + stepperDir + "/" + image + "-off.png";
    }
        
    else if (event.type = "mousedown") {
        event.srcElement.src = "Images/stepper/" + stepperDir + "/" + image + "-on.png";
        if (event.srcElement.name.indexOf("transpose") != -1) {
            var target = transpose + shiftAmount;
            if (-36 <= target && target <= 36) {
                transpose = target;
                document.configuration.transposeValue.value = transpose;
            }
        } else {
            var target = keyShift + shiftAmount;
            if (-36 <= target && target <= 36) {
                keyShift += shiftAmount;
                document.configuration.keyshiftValue.value = keyShift;
            }
        }
    } 
}

function getCurrentVersion() {

    // read the info file
    var request = new XMLHttpRequest();
    request.open("GET", "Info.plist", false);
    request.send(null);
    
    // parse the version key
    var version = request.responseText.replace(/\s/g, "");
    version = version.replace(/.*CFBundleShortVersionString<\/key><string>([^<]*)<\/string>.*/, "$1");
    return version;
}    

function checkForUpdate() {
    updateRequest = new XMLHttpRequest();
    updateRequest.onreadystatechange=checkForUpdateResult;
    updateRequest.open("GET", "http://www.mit.edu/~jwhite1/piano/current-version.txt", true);
    updateRequest.send(null);
}

function checkForUpdateResult() {
    if (updateRequest.readyState != 4) return;
    if (updateRequest.status != 200) return;
    if (updateRequest.responseText == null) return;
    if (updateRequest.responseText.replace(/\s/g, "") != getCurrentVersion()) {
        // update is available
    } 
}